==> lkrg-0.5.tar.gz: lkrg-0.5/src/modules/exploit_detection/syscalls/p_sel_write_enforce/p_sel_write_enforce.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept 'sel_write_enforce' function
 *
 * Notes:
 *  - Intercept SELinux state modifications
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 13.XI.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */


#ifdef CONFIG_SECURITY_SELINUX

#include "../../../../p_lkrg_main.h"

char p_sel_write_enforce_kretprobe_state = 0x0;

static struct kretprobe p_sel_write_enforce_kretprobe = {
    .kp.symbol_name = "sel_write_enforce",
    .handler = p_sel_write_enforce_ret,
    .entry_handler = p_sel_write_enforce_entry,
    .data_size = sizeof(struct p_sel_write_enforce_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

/*
 * x86-64 syscall ABI:
 *  *rax - syscall_number
 *    rdi - 1st argument
 *    rsi - 2nd argument
 *    rdx - 3rd argument
 *    rcx - 4th argument
 *
 *    r8  - 5th one
 *    r9  - 6th one
 */

int p_sel_write_enforce_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_sel_write_enforce_entry>\n");

   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - validate 'off' flag
      p_ed_is_off_off_wrap(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);

   p_ed_enforce_validation();

   // lock shadow SELinux updates
   mutex_lock(&p_ed_guard_globals.p_selinux_lock);

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_sel_write_enforce_entry>\n");

   return 0x0;
}


int p_sel_write_enforce_ret(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_sel_write_enforce_ret>\n");

   // Success ?
   if (!IS_ERR((void *)p_regs->ax)) {
      // track down new SELinux information
#ifdef CONFIG_SECURITY_SELINUX_DEVELOP
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4,17,0)
      p_ed_guard_globals.p_selinux.p_selinux_state.enforcing = p_selinux_state->enforcing;
 #else
      p_ed_guard_globals.p_selinux.p_selinux_enforcing = *p_selinux_enforcing;
 #endif
#endif
      p_ed_guard_globals.p_selinux.p_selinux_enabled = *p_selinux_enabled;
   }

   // unlock shadow SELinux updates
   mutex_unlock(&p_ed_guard_globals.p_selinux_lock);

   p_ed_enforce_validation();

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_sel_write_enforce_ret>\n");

   return 0x0;
}


int p_install_sel_write_enforce_hook(void) {

   int p_ret;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_sel_write_enforce_hook>\n");

   if ( (p_ret = register_kretprobe(&p_sel_write_enforce_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_sel_write_enforce_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_install_sel_write_enforce_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_sel_write_enforce_kretprobe.kp.symbol_name,
               p_sel_write_enforce_kretprobe.kp.addr);
   p_sel_write_enforce_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_install_sel_write_enforce_hook_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_sel_write_enforce_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_sel_write_enforce_hook(void) {

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_sel_write_enforce_hook>\n");

   if (!p_sel_write_enforce_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_sel_write_enforce_kretprobe.kp.symbol_name,
                  p_sel_write_enforce_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_sel_write_enforce_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_sel_write_enforce_kretprobe.kp.symbol_name,
                  p_sel_write_enforce_kretprobe.kp.addr,
                  p_sel_write_enforce_kretprobe.nmissed);
      p_sel_write_enforce_kretprobe_state = 0x0;
   }

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_sel_write_enforce_hook>\n");
}

#endif

==> lkrg-0.8.tar.gz: lkrg-0.8/src/modules/exploit_detection/syscalls/p_sel_write_enforce/p_sel_write_enforce.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept 'sel_write_enforce' function
 *
 * Notes:
 *  - Intercept SELinux state modifications
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 13.XI.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */


#ifdef CONFIG_SECURITY_SELINUX

#include "../../../../p_lkrg_main.h"

char p_sel_write_enforce_kretprobe_state = 0x0;

static struct kretprobe p_sel_write_enforce_kretprobe = {
    .kp.symbol_name = "sel_write_enforce",
    .handler = p_sel_write_enforce_ret,
    .entry_handler = p_sel_write_enforce_entry,
    .data_size = sizeof(struct p_sel_write_enforce_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

/*
 * x86-64 syscall ABI:
 *  *rax - syscall_number
 *    rdi - 1st argument
 *    rsi - 2nd argument
 *    rdx - 3rd argument
 *    rcx - 4th argument
 *
 *    r8  - 5th one
 *    r9  - 6th one
 */

int p_sel_write_enforce_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_sel_write_enforce_entry>\n");

   p_tasks_read_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      // This process is on the ED list - validate 'off' flag
      p_ed_is_off_off_wrap(p_tmp);
   }
   p_tasks_read_unlock(&p_flags);

   p_ed_enforce_validation();

   // lock shadow SELinux updates
   mutex_lock(&p_ed_guard_globals.p_selinux_lock);

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_sel_write_enforce_entry>\n");

   return 0x0;
}


int p_sel_write_enforce_ret(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_sel_write_enforce_ret>\n");

   // Success ?
   if (!IS_ERR((void *)p_regs_get_ret(p_regs))) {
      // track down new SELinux information
#ifdef CONFIG_SECURITY_SELINUX_DEVELOP
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4,17,0)
      p_ed_guard_globals.p_selinux.p_selinux_state.enforcing = P_SYM(p_selinux_state)->enforcing;
 #else
      p_ed_guard_globals.p_selinux.p_selinux_enforcing = *P_SYM(p_selinux_enforcing);
 #endif
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 6, 0)
      p_ed_guard_globals.p_selinux.p_selinux_enabled = *P_SYM(p_selinux_enabled);
#endif
   }

   // unlock shadow SELinux updates
   mutex_unlock(&p_ed_guard_globals.p_selinux_lock);

   p_ed_enforce_validation();

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_sel_write_enforce_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(sel_write_enforce)

#endif
